# NTray - A tray notification tool for the Nagios Monitoring System
# Copyright (C) 2002 Robert M. Wagner (rob@robandsara.com)
# Licensed under the GPL (see included LICENSE file for full license)
# 
# Nagios is a registered trademark of Ethan Galstad. (nagios@nagios.org)
# You can visit the Nagios home page at http://www.nagios.org
#
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

my $version = ".90d"; #Just for reference :)

use XML::Simple;
use Win32::GUI;
use Win32::Shell;
use LWP::UserAgent;
use Win32::Sound;
use Strict;


#Declare this variable so it passes things okay later
my $refresh_time = 0;

#We'll get this from a config file/ini file/registry later
my ($popup_xpos,$popup_ypos) = (150,200);

#parse command-line options
my @commandline = @ARGV;
while (scalar(@commandline) > 0) {
   print "CMD = $commandline[0]\n";
   if (lc($commandline[0]) eq "--help") {
      die (Win32::GUI::MessageBox(0, "NTray $version Usage:\nNTray.exe \[-c d:\\path\\config.xml\] \[-w seconds_to_refresh\]\n\nExample:\nNTray.exe -c c:\\etc\\ntrayconfig.xml -w 60\n", "NTray Help"));
   }
   if ($commandline[0] eq "-c") {
      $config_file = $commandline[1];
 
      shift @commandline;
      shift @commandline;
   }
   if ($commandline[0] eq "-w") {
      $refresh_time = $commandline[1] * 1000;
      print "Refresh is: $refresh_time\n";
      shift @commandline;
      shift @commandline;
   }

}

#Default XML config file (if not specified on command line)
if ($config_file eq "") {
     print "Config file is: $config_file\n";
	$config_file = "./config.xml";
}

#Read in the XML Config File
my $xml = XML::Simple->new();
my $config;

read_configfile($config_file);


#Create our forms and icons
create_Objects();


#Create the initial icon
my ($status_color,$status_text) = get_status($config->{CONFIG}->{BASIC}->{CGI_URL},$config->{CONFIG}->{AUTHENTICATION}->{AUTH_REQUIRED},$config->{CONFIG}->{AUTHENTICATION}->{AUTH_USERNAME},$config->{CONFIG}->{AUTHENTICATION}->{AUTH_PASSWORD});
$last_color = $status_color;
create_icon($status_color,$status_text);

#Run the Dialog loop, which handles all the event driven stuff in the program
Win32::GUI::Dialog();
write_configfile($config_file);

sub create_Objects {
	create_Main() or die ("Unable to create Main form\n");
	create_Popup($status_color) or die ("Unable to create Pupup form\n");
	create_Timer() or die ("Unable to create Timer on Main form\n");
	define_Icons() or die ("Unable to create Icon objects\n");
}

sub create_Main {
	#Create the Main form. We will attach the timer and icon to this.
	$main = Win32::GUI::Window->new(-name => 'Main', -text => 'NTray',
                                -width => 10, -height => 10,
                                -left => 10, -top => 10);

	1;
}

sub create_Popup {
	my ($color) = @_;
	#Create the Popup Window Form
	$popup = Win32::GUI::Window->new(-name => 'Popup', -text => 'NTray Popup Notification',
                                -width => 440, -height => 240,
                                -left => $config->{CONFIG}->{POPUPS}->{POPUP_XPOS}, 
                                -top => $config->{CONFIG}->{POPUPS}->{POPUP_YPOS},
                                -icon => $$color);
	1;
}

sub create_Timer {
	#Create the timer attached to the 'Main' form
	if (! $refresh_time ) {
	   print "Setting refresh time from the config file\n";
	   $refresh_time = $config->{CONFIG}->{BASIC}->{REFRESH_TIMER} * 1000; #Convert refresh time to milliseconds
	}
	$countdown = $main->AddTimer(Countdown,$refresh_time);

	1;
}

sub define_Icons {
	#Define the Icon Objects
	$green = new Win32::GUI::Icon("$config->{CONFIG}->{BASIC}->{NTRAY_PATH}/green.ico");
	$yellow = new Win32::GUI::Icon("$config->{CONFIG}->{BASIC}->{NTRAY_PATH}/yellow.ico");
	$red = new Win32::GUI::Icon("$config->{CONFIG}->{BASIC}->{NTRAY_PATH}/red.ico");
	$unknown = new Win32::GUI::Icon("$config->{CONFIG}->{BASIC}->{NTRAY_PATH}/unknown.ico");

	1;
}


sub read_configfile {
	#I love XML, don't hate me :)
	my $config_file = $_[0];

	if (-e "$config_file") {
		$config = $xml->XMLin("$config_file", keeproot => 1);
	} else {
		die "Error: Program config file not found ($config_file)\n";
	}
}

sub write_configfile {
	#I love XML, don't hate me :)
	my $config_file = $_[0];

	if (-e "$config_file") {
		open (OUTFILE,">$config_file");
		print OUTFILE $xml->XMLout($config, keeproot => 1, noattr => 1);
		close OUTFILE;
	} else {
		die "Error: Program config file not found ($config_file)\n";
	}
}


sub create_icon {
	#Initially create the Tray Icon (using $color.ico)
	my ($color,$text) = @_;

	$main->AddNotifyIcon(
		-name => "TrayIcon",
		-tip  => "   Status: $text",
		-icon => $$color,
		-id   => 1 );


	1; #Don't Die
}

sub change_icon {
	#Change the Tray Icon to a different icon ($color.ico)
	my ($color,$text) = @_;
	$popup->ChangeIcon($$color);
	
	Win32::GUI::NotifyIcon::Modify(
		$main, 
		-id => 1, 
		-icon => $$color, 
		-tip => "   Status: $text" );
	1; #Don't Die
}

sub TrayIcon_Click {
	#Single (left) Click on the Tray Icon (opens Nagios in your default Browser)
	Win32::Shell::Execute("open","$config->{CONFIG}->{BASIC}->{NAGIOS_URL}",undef,undef, "SW_SHOWNORMAL");
	1;
}


sub Popup_Terminate {
	print "Hiding Popup Window\n";
	($status_color,$status_text) = get_status($config->{CONFIG}->{BASIC}->{CGI_URL},$config->{CONFIG}->{AUTHENTICATION}->{AUTH_REQUIRED},$config->{CONFIG}->{AUTHENTICATION}->{AUTH_USERNAME},$config->{CONFIG}->{AUTHENTICATION}->{AUTH_PASSWORD});
	$config->{CONFIG}->{POPUPS}->{POPUP_XPOS} = $popup->Left();
	$config->{CONFIG}->{POPUPS}->{POPUP_YPOS} = $popup->Top();
	create_Popup($status,$status_color);
}


sub TrayIcon_RightClick {
	#Right Click on the Tray Icon
	my ($x,$y) = Win32::GUI::GetCursorPos();
    
	my $menu = new Win32::GUI::Menu(
		"tray"      => "tray",
		" > &Nagios" => "Nagios",
		" > -"	    => 0,
		" > &Status" => "Status",
		" > &About"  => "About",
		" > E&xit"   => "Exit",
		);

	$main->TrackPopupMenu($menu->{tray}, $x, $y);
	1; #Don't Die
}


sub Exit_Click {
	#Click on the 'Exit' Menu Option
	-1; #Exit Win32::Gui::Dialog loop
}


sub About_Click {
	#Click on the 'About' Menu Option
	Win32::GUI::MessageBox(0, "Ntray!\n\n(c)2002 by Rob Wagner (rob\@robandsara.com)\n\nLicensed Under the GPL\n", "About NTray");
	1;
}


sub Nagios_Click {
	#Click on the 'Nagios' Menu Option	
	Win32::Shell::Execute("open","$config->{CONFIG}->{BASIC}->{NAGIOS_URL}",undef,undef, "SW_SHOWNORMAL");
	1;
}

sub Status_Click {
	#Click on the 'Status' Menu Option	
($status_color,$status_text) = get_status($config->{CONFIG}->{BASIC}->{CGI_URL},$config->{CONFIG}->{AUTHENTICATION}->{AUTH_REQUIRED},$config->{CONFIG}->{AUTHENTICATION}->{AUTH_USERNAME},$config->{CONFIG}->{AUTHENTICATION}->{AUTH_PASSWORD});
	my $color = $status_color;
	$popup->ChangeIcon($$color);
	show_Popup($status_color,$status_text);
	1;
}



sub Countdown_Timer {
	($status_color,$status_text) = get_status($config->{CONFIG}->{BASIC}->{CGI_URL},$config->{CONFIG}->{AUTHENTICATION}->{AUTH_REQUIRED},$config->{CONFIG}->{AUTHENTICATION}->{AUTH_USERNAME},$config->{CONFIG}->{AUTHENTICATION}->{AUTH_PASSWORD});
	
	if ($last_color ne $status_color) {
		#Change the icon color if needed
		change_icon($status_color,$status_text);
		
		if ($config->{CONFIG}->{POPUPS}->{ENABLE_POPUPS} =~ /true/i ) {
			show_Popup($status_color,$status_text);
		}

		if ((-e "$status_color.wav") && ($config->{CONFIG}->{SOUND}->{ENABLE_SOUND} =~ /true/i) ) {
			print "Attempting to play $status_color.wav\n";
			#If sound is enabled and the sound file exists...
			Win32::Sound::Play("$status_color.wav");
			Win32::Sound::Stop();
			
		}
	}
	$last_color = $status_color;
	print "Color is $status_color\n";
	1;
}

sub get_status {
	#Get the status by doing a quick check of the status.cgi page
	my $color = "unknown";
	
	my $ua = LWP::UserAgent->new;
	my $url = $config->{CONFIG}->{BASIC}->{CGI_URL} . "/status.cgi?hostgroup=all&style=summary";
	my $req = HTTP::Request->new(GET => $url);
	if ($config->{CONFIG}->{AUTHENTICATION}->{AUTH_REQUIRED} =~ /true/i) {
		$req->authorization_basic($config->{CONFIG}->{AUTHENTICATION}->{AUTH_USERNAME},$config->{CONFIG}->{AUTHENTICATION}->{AUTH_PASSWORD});
	}
	my $content = $ua->request($req)->as_string;


	if ($content =~ /last updated/i) {
		$color = "green";
	}

	if ($content =~ /WARNING/) {
		$color = "yellow";
	}

	if ($content =~ /CRITICAL/) {
		$color = "red";
	}

	
	$text = "No extended status available at this time";
	
	return ($color,$text);
}

sub show_Popup {
	my ($color,$text) = @_;
	print "Showing Popup Window\n";
	$popup->Hide();

	
	if (! defined ($popup->{'gbStatus'})) {
		$popup->AddGroupbox(
		       -text    => "Nagios Status: $color",
		       -name    => "gbStatus",
		       -left    => 4,
		       -top     => 4,
		       -width   => 420,
		       -height  => 200,
		);
	} else {
		$popup->{'gbStatus'}->Text("Nagios Status: $color");
	}

	if (! defined ($popup->{'lblStatus'})) {
		$popup->AddLabel(
		       -text    => "$text\n\n",
		       -name    => "lblStatus",
		       -left    => 14,
		       -top     => 20,
		       -width   => 400,
		       -height  => 180,
		);
	} else {
		$popup->{'lblStatus'}->Text("$text");
	}

	$popup->Show();

}